/* Sistemas Integrados TECNISA
   Compras e Suprimentos

Nome: 8950_SUPSTPS_NECESSIDADE_OL.sql

Descrio:
Clculo da consolidao de solicitao de compras para o operador logstico

@P_CDFILIAL    - Operador Logstico.
@P_DTINICIAL   - Data inicial.
@P_DTFINAL     - Data final.
@P_CDPRODINI   - Cdigo de produto inicial.
@P_CDPRODFIN   - Cdigo de produto final.
@P_PEACRESCIMO - Percentual de acrscimo.
@P_CDOPERADOR  - Cdigo do operador que solicitou o clculo.
@P_NRSEQCONSOL - Nmero sequencial a ser utilizado para a consolidao de solicitao de compras - OL.
@P_CDFILLANC   - Filial de Processamento

Criao:
09/02/2007 Marcelo Alexsandro Souza Silva
*/

IF EXISTS (SELECT * FROM sysobjects O, sysusers U
           WHERE o.name = 'NECESSIDADE_OL'
             and type = 'P'
             and u.uid = o.uid
             and u.name = USER)
   DROP PROCEDURE NECESSIDADE_OL;
GO

CREATE PROCEDURE NECESSIDADE_OL
   @P_CDFILIAL        VARCHAR(4),
   @P_DTINICIAL       DATETIME,
   @P_DTFINAL         DATETIME,
   @P_CDPRODINI       VARCHAR(20),
   @P_CDPRODFIN       VARCHAR(20),
   @P_PEACRESCIMO     NUMERIC,
   @P_CDOPERADOR      VARCHAR(12),
   @P_NRSEQCONSOL     VARCHAR(10),
   @P_CDFILLANC       VARCHAR(4),
   @P_PRIORIZAREST    VARCHAR(1),
   @P_PREVISAOCONSUMO VARCHAR(1),
   @P_GERASOLOPERLOG  VARCHAR(1),
   @P_ROWSAFFECT      VARCHAR(1) OUTPUT
AS
   DECLARE @V_NOVADATA            DATETIME;
   DECLARE @V_DTENTREGA           DATETIME;
   DECLARE @V_DTMESENTREGA        DATETIME;
   DECLARE @V_DTPROCESS           DATETIME;
   DECLARE @V_DTPROCESS_MAIS_UM   DATETIME;
   DECLARE @V_ITEMSOLI_DTPROGENTR DATETIME;

   DECLARE @V_CDPRODINI            VARCHAR(20);
   DECLARE @V_CDPRODFIN            VARCHAR(20);
   DECLARE @W_N                    VARCHAR(1);
   DECLARE @W_S                    VARCHAR(1);
   DECLARE @W_P                    VARCHAR(1);
   DECLARE @W_A                    VARCHAR(1);
   DECLARE @W_I                    VARCHAR(1);
   DECLARE @W_4                    VARCHAR(1);
   DECLARE @W_5                    VARCHAR(1);
   DECLARE @W_6                    VARCHAR(1);
   DECLARE @V_ITEM_CONSOLIDADO     VARCHAR(1);
   DECLARE @V_PRIMEIRO_REGISTRO    VARCHAR(1);
   DECLARE @V_NRLIMISEGCAD         VARCHAR(3);
   DECLARE @V_EMBALAGEM_CDPRODUTO  VARCHAR(10);
   DECLARE @V_EMBALAGEM_CDARVPROD  VARCHAR(20);
   DECLARE @V_EMBALAGEM_CDPERIENTR VARCHAR(2);
   DECLARE @V_ITEMSOLI_CDFILSOLI   VARCHAR(4);
   DECLARE @V_ITEMSOLI_CDPRODUTO   VARCHAR(10);
   DECLARE @V_ITEMSOLI_CDARVPROD   VARCHAR(20);
   DECLARE @V_ESTOQUE_CDPRODARVOL  VARCHAR(10);
   DECLARE @V_ESTOQUE_CDMARCA      VARCHAR(5);
   DECLARE @V_ESTOQUE_CDPRODESTO   VARCHAR(10);
   DECLARE @V_IDUTILLOTE           VARCHAR(1);
   DECLARE @V_CDOPERADOR           VARCHAR(12);

   DECLARE @W_ZERO         INT;
   DECLARE @W_UM           INT;

   DECLARE @V_CONTADOR              DOUBLE PRECISION;
   DECLARE @V_QTDEPRODUTO           DOUBLE PRECISION;
   DECLARE @V_NRDIASEG              DOUBLE PRECISION;
   DECLARE @V_QTDERESTANTE          DOUBLE PRECISION;
   DECLARE @V_AUXQTDERESTANTE       DOUBLE PRECISION;
   DECLARE @V_QTESTOQUENECCMP       DOUBLE PRECISION;
   DECLARE @V_QTDIAESTOQUE          DOUBLE PRECISION;
   DECLARE @V_VALRESTO              DOUBLE PRECISION;
   DECLARE @V_VALRBRUT              DOUBLE PRECISION;
   DECLARE @V_QTDE_ENTRADA          DOUBLE PRECISION;
   DECLARE @V_QTDE_SAIDA            DOUBLE PRECISION;
   DECLARE @V_EMBALAGEM_VRFATOCONV  DOUBLE PRECISION;
   DECLARE @V_EMBALAGEM_VRARREDCMP  DOUBLE PRECISION;
   DECLARE @V_ITEMSOLI_QTSOLIPROD   DOUBLE PRECISION;
   DECLARE @V_CDFILANTERIOR         VARCHAR(4);
   DECLARE @V_CDPRODANTERIOR        VARCHAR(10);
   DECLARE @V_IDUTEMPENCONSOL       VARCHAR(1);

   DECLARE @V_LISTA_ESTOQUE         VARCHAR(20);
   DECLARE @V_COUNT                 DOUBLE PRECISION;
   DECLARE @V_LISTA_ESTOQUE_VAZIA   VARCHAR(1);

   DECLARE @V_PROD_ESTOQUE_CDPRODARVOL VARCHAR(10);
   DECLARE @V_PROD_ESTOQUE_CDPRARVORIG VARCHAR(10);

   DECLARE @V_PRXNECECOMP VARCHAR(20);

   DECLARE @V_FILIALNECECOMP_CDFILIAL VARCHAR(4);

   DECLARE @V_DATAATUAL DATETIME;

   DECLARE @V_DTFINAL              DATETIME;

   DECLARE @V_QTESTOQUESEGURANCA   DOUBLE PRECISION;
   DECLARE @V_QTSEGURANCA          DOUBLE PRECISION;
   DECLARE @V_MAX_LIMISEGU         DOUBLE PRECISION;
BEGIN
   SET @W_N                = 'N';
   SET @W_S                = 'S';
   SET @W_P                = 'P';
   SET @W_A                = 'A';
   SET @W_I                = 'I';
   SET @W_4                = '4';
   SET @W_5                = '5';
   SET @W_6                = '6';
   SET @V_ITEM_CONSOLIDADO = 'N';
   SET @W_ZERO             = 0;
   SET @W_UM               = 1;

   SET @P_ROWSAFFECT       = @W_N;
   SET @V_CDPRODINI = @P_CDPRODINI;
   SET @V_CDPRODFIN = (SUBSTRING(@P_CDPRODFIN+(REPLICATE('Z', 20)), 1, 20));

   SET @V_QTESTOQUESEGURANCA   = @W_ZERO;
   SET @V_QTSEGURANCA          = @W_ZERO;

   SELECT
      @V_DTPROCESS = (CASE
                        ISNULL(MAX(P.IDUTILDTPROC), @W_N)
                        WHEN @W_S THEN ISNULL(MAX(PF.DTPROCESSA), GETDATE())
                        ELSE GETDATE()
                      END
                      ),
      @V_IDUTILLOTE = ISNULL(MAX(PF.IDUTILLOTE), @W_N)
   FROM
      PARAMFILIAL PF, PARAMGERAL P
   WHERE PF.CDFILIAL = @P_CDFILIAL;
   SET @V_DTPROCESS_MAIS_UM = CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE() + @W_UM, 103),103);

   SELECT
      @V_IDUTEMPENCONSOL = ISNULL(MAX(IDUTEMPENCONSOL), @W_N)
   FROM
      FILIAL, PARASUPRI
   WHERE PARASUPRI.CDFILIAL = ISNULL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)
     AND FILIAL.CDFILIAL    = @P_CDFILIAL;

   IF @P_CDOPERADOR IS NOT NULL BEGIN
      INSERT INTO CONSSOLOL
        (NRSEQUCONSOL, DTINICONSSOLOL, DTFINCONSSOLOL,
         IDSITUCONSSOL, DTCONSSOLOL, CDOPERADOR, CDFILIOPELOG, IDEMBESTCONSOL,
         IDCONSPREVCONOL, CDPRODINICONSOL, CDPRODFINCONSOL)
      VALUES
        (@P_NRSEQCONSOL, @P_DTINICIAL, @P_DTFINAL,
         @W_P, GETDATE(), @P_CDOPERADOR, @P_CDFILIAL, @P_PRIORIZAREST,
         @P_PREVISAOCONSUMO, @P_CDPRODINI, @P_CDPRODFIN);
   END
   ELSE BEGIN
       SELECT
          @V_CDOPERADOR = MAX(CDOPERADOR)
       FROM CONSSOLOL
       WHERE NRSEQUCONSOL = @P_NRSEQCONSOL;
   END;
   SET @V_CDFILANTERIOR  = ' ';
   SET @V_CDPRODANTERIOR = ' ';
   SET @V_PRXNECECOMP    = NULL;
   IF @P_PREVISAOCONSUMO = @W_N BEGIN
      DECLARE C_ITEMSOLI_AUX CURSOR FOR
         SELECT
            ITEMSOLI.CDFILSOLI,
            PRODESTO.CDPRODUTO,
            PRODESTO.CDARVPROD,
            ITEMSOLI.DTPROGENTR,
            SUM(ITEMSOLI.QTSOLIPROD*PRODUTO.VRFATOCONV) QTSOLIPROD
         FROM ITEMSOLI, PRODUTO, PRODUTO PRODESTO
         WHERE ITEMSOLI.DTPROGENTR BETWEEN @P_DTINICIAL AND @P_DTFINAL+(CASE @P_GERASOLOPERLOG
                                                                           WHEN 'S'
                                                                           THEN 30
                                                                           ELSE 0
                                                                        END)
           AND PRODESTO.CDARVPROD  BETWEEN @V_CDPRODINI AND @V_CDPRODFIN
           AND ((ITEMSOLI.CDFILIOPELOG  = @P_CDFILIAL) OR (@P_CDOPERADOR IS NULL))
           AND ITEMSOLI.CDFILSOLI    <> @P_CDFILIAL
           AND ISNULL(ITEMSOLI.IDSOLIAPROV, @W_S) = @W_S
           AND ISNULL(ITEMSOLI.IDITMCONS, @W_N) = @W_N
           AND PRODUTO.CDPRODUTO      = ITEMSOLI.CDPRODUTO
           AND PRODUTO.CDPRODESTO     = PRODESTO.CDPRODUTO
           AND ITEMSOLI.NRSEQUCONSOL  IS NULL
         GROUP BY
            ITEMSOLI.CDFILSOLI,
            PRODESTO.CDPRODUTO,
            PRODESTO.CDARVPROD,
            ITEMSOLI.DTPROGENTR
         ORDER BY
            ITEMSOLI.CDFILSOLI,
            PRODESTO.CDPRODUTO,
            ITEMSOLI.DTPROGENTR;
   END
   ELSE BEGIN
      EXEC NOVO_CODIGO 'NECECOMP', @P_SEQUENCIAL = @V_PRXNECECOMP OUTPUT, @P_QTDE = 1;
      SET @V_PRXNECECOMP = SUBSTRING(@V_PRXNECECOMP, 11, 10);
      DECLARE C_FILIALNECECOMP CURSOR FOR
         SELECT DISTINCT CDFILIAL
         FROM PARAMPRODSUP
         WHERE CDFILIOPELOG = @P_CDFILIAL;
      OPEN C_FILIALNECECOMP;
      FETCH NEXT FROM C_FILIALNECECOMP INTO @V_FILIALNECECOMP_CDFILIAL;
      WHILE (@@FETCH_STATUS = 0) BEGIN
         SET @V_DATAATUAL = GETDATE();
         SET @V_DTFINAL    = @P_DTFINAL+(CASE @P_GERASOLOPERLOG
                                   WHEN 'S'
                                   THEN 30
                                   ELSE 0
                                END);
         EXEC NECESSIDADE_DE_COMPRAS @V_FILIALNECECOMP_CDFILIAL, /*P_CDFILIAL*/
                                     NULL,                       /*P_TCDFILIAL*/
                                     NULL,                       /*P_CDGRUPFILI*/
                                     @V_DTPROCESS,               /*P_DTINICIAL*/ /*sol 152483 - data de processamento*/
                                     @V_DTFINAL,                 /*P_DTFINAL*/
                                     @P_CDPRODINI,               /*P_CDPRODINI*/
                                     @P_CDPRODFIN,               /*P_CDPRODFIN*/
                                     0,                          /*P_PEACRESCIMO*/
                                     @P_CDOPERADOR,              /*P_CDOPERADOR*/
                                     @V_DATAATUAL,               /*P_DTHRCALCNC*/
                                     @W_N,                       /*P_IDQTDEZERO*/
                                     @V_PRXNECECOMP,             /*P_NRNECECOMP*/
                                     @W_N                        /*P_EXTRA*/;
         FETCH NEXT FROM C_FILIALNECECOMP INTO @V_FILIALNECECOMP_CDFILIAL;
      END;
      DECLARE C_ITEMSOLI_AUX CURSOR FOR
         SELECT
            TRABNECECOMP.CDFILNECECMP,
            PRODESTO.CDPRODUTO,
            PRODESTO.CDARVPROD,
            TRABNECECOMP.DTENTRNECCMP,
            SUM(TRABNECECOMP.QTPRCNVCOMPR*PRODUTO.VRFATOCONV) QTSOLIPROD
         FROM TRABNECECOMP, PRODUTO, PRODUTO PRODESTO
         WHERE TRABNECECOMP.DTENTRNECCMP BETWEEN @P_DTINICIAL AND @P_DTFINAL
           AND PRODESTO.CDARVPROD  BETWEEN @V_CDPRODINI AND @V_CDPRODFIN
           AND PRODUTO.CDPRODUTO          = TRABNECECOMP.CDPRODNECCMP
           AND PRODUTO.CDPRODESTO         = PRODESTO.CDPRODUTO
           AND TRABNECECOMP.CDFILNECECMP <> @P_CDFILIAL
           AND (TRABNECECOMP.NRNECECOMP   = @V_PRXNECECOMP)
           AND (TRABNECECOMP.IDMARCNECCMP = @W_S)
           AND (TRABNECECOMP.CDFILIOPELOG = @P_CDFILIAL)
         GROUP BY
            TRABNECECOMP.CDFILNECECMP,
            PRODESTO.CDPRODUTO,
            PRODESTO.CDARVPROD,
            TRABNECECOMP.DTENTRNECCMP
         ORDER BY
            TRABNECECOMP.CDFILNECECMP,
            PRODESTO.CDPRODUTO,
            TRABNECECOMP.DTENTRNECCMP;
   END;
   OPEN C_ITEMSOLI_AUX;
   FETCH NEXT FROM C_ITEMSOLI_AUX INTO @V_ITEMSOLI_CDFILSOLI,
                                   @V_ITEMSOLI_CDPRODUTO,
                                   @V_ITEMSOLI_CDARVPROD,
                                   @V_ITEMSOLI_DTPROGENTR,
                                   @V_ITEMSOLI_QTSOLIPROD;
   WHILE (@@FETCH_STATUS = 0) BEGIN
      IF (@V_CDFILANTERIOR = ' ') AND (@V_CDPRODANTERIOR = ' ') BEGIN
         SET @V_CDFILANTERIOR  = @V_ITEMSOLI_CDFILSOLI;
         SET @V_CDPRODANTERIOR = @V_ITEMSOLI_CDPRODUTO;
      END
      ELSE BEGIN
         IF @V_IDUTEMPENCONSOL = @W_S BEGIN
            IF (@V_CDFILANTERIOR <> @V_ITEMSOLI_CDFILSOLI) OR (@V_CDPRODANTERIOR <> @V_ITEMSOLI_CDPRODUTO) BEGIN
               IF @P_CDOPERADOR IS NOT NULL BEGIN
                  EXEC EMPENHO_OL @P_NRSEQCONSOL, @P_CDFILIAL, @V_CDFILANTERIOR, @V_CDPRODANTERIOR, @P_CDOPERADOR, @V_PRXNECECOMP;
               END
               ELSE BEGIN
                  EXEC EMPENHO_OL @P_NRSEQCONSOL, @P_CDFILIAL, @V_CDFILANTERIOR, @V_CDPRODANTERIOR, @V_CDOPERADOR, @V_PRXNECECOMP;
               END;
               SET @V_CDFILANTERIOR  = @V_ITEMSOLI_CDFILSOLI;
               SET @V_CDPRODANTERIOR = @V_ITEMSOLI_CDPRODUTO;
            END;
         END;
      END;

      SET @V_QTDERESTANTE      = @V_ITEMSOLI_QTSOLIPROD;
      SET @V_QTDEPRODUTO       = @W_ZERO;
      SET @V_ITEM_CONSOLIDADO  = @W_N;
      SET @V_PRIMEIRO_REGISTRO = @W_S;
      IF @P_PRIORIZAREST = @W_N BEGIN
         DECLARE C_EMBALAGEM CURSOR FOR
            SELECT
               PRODUTO.CDPRODUTO,
               PRODUTO.CDARVPROD,
               ISNULL(PRODUTO.VRFATOCONV, @W_UM) VRFATOCONV,
               ISNULL(PRODUTO.VRARREDCMP, @W_ZERO) VRARREDCMP,
               PERFILPROD.CDPERIENTR
            FROM PRODUTO, PRODUTO PRODPERIENTR, PERFILPROD
            WHERE ISNULL(PRODUTO.IDPRCMPATIVO, @W_S)= @W_S
               AND PRODUTO.CDPRODESTO     = @V_ITEMSOLI_CDPRODUTO
               AND PERFILPROD.CDPRODUTO   = PRODPERIENTR.CDPRODUTO
               AND PERFILPROD.CDFILIAL    = @P_CDFILIAL
               AND EXISTS (SELECT 1
                           FROM PRODFILI
                           WHERE PRODFILI.CDFILIAL = @P_CDFILIAL
                             AND PRODFILI.CDPRODUTO = PRODUTO.CDPRODUTO
                             AND ISNULL(PRODFILI.IDPRODATIV, @W_A)=@W_A)
               AND PRODPERIENTR.CDARVPROD = (SELECT
                                                MAX(B.CDARVPROD)
                                             FROM
                                                PERFILPROD A,
                                                PRODUTO B
                                             WHERE A.CDPRODUTO = B.CDPRODUTO
                                              AND  A.CDFILIAL  = @P_CDFILIAL
                                              AND  CHARINDEX(B.CDARVPROD, PRODUTO.CDARVPROD) = @W_UM)
            ORDER BY
               ISNULL(PRODUTO.VRFATOCONV, 1)*ISNULL(PRODUTO.VRARREDCMP, 1) DESC;
      END
      ELSE BEGIN
         SET @V_LISTA_ESTOQUE_VAZIA = @W_S;
         EXEC NOVO_CODIGO 'LISTA', @P_SEQUENCIAL = @V_LISTA_ESTOQUE, @P_QTDE = 1;
         SET @V_LISTA_ESTOQUE = SUBSTRING(@V_LISTA_ESTOQUE, 11, 10);
         DECLARE C_PROD_ESTOQUE CURSOR FOR
            SELECT RELACPRODOL.CDPRODARVOL, RELACPRODOL.CDPRARVORIG
            FROM PRODUTO, RELACPRODOL
            WHERE PRODUTO.CDPRODESTO = @V_ITEMSOLI_CDPRODUTO
              AND RELACPRODOL.CDPRARVORIG = PRODUTO.CDPRODUTO;
         OPEN C_PROD_ESTOQUE;
         FETCH NEXT FROM C_PROD_ESTOQUE INTO @V_PROD_ESTOQUE_CDPRODARVOL,
                                             @V_PROD_ESTOQUE_CDPRARVORIG;
         WHILE (@@FETCH_STATUS = 0) BEGIN
            SET @V_COUNT = 0;
            SELECT @V_COUNT = COUNT(1)
            FROM PRODUTO
            WHERE PRODUTO.CDPRODUTO = @V_PROD_ESTOQUE_CDPRODARVOL
              AND PRODUTO.CDPRODUTO = PRODUTO.CDPRODESTO;
            IF @V_COUNT > 0 BEGIN
               IF (TEKNISA.CALC_ESTOQ_CONS(@P_CDFILIAL, @V_PROD_ESTOQUE_CDPRODARVOL, @V_DTPROCESS_MAIS_UM) > 0) BEGIN
                  SET @V_LISTA_ESTOQUE_VAZIA = @W_N;
                  INSERT INTO LISTA
                     (NRLISTA, CDLISTA)
                  VALUES
                     (@V_LISTA_ESTOQUE, @V_PROD_ESTOQUE_CDPRARVORIG);
               END;
            END;
            FETCH NEXT FROM C_PROD_ESTOQUE INTO @V_PROD_ESTOQUE_CDPRODARVOL,
                                                @V_PROD_ESTOQUE_CDPRARVORIG;
         END;
         CLOSE C_PROD_ESTOQUE;
         DEALLOCATE C_PROD_ESTOQUE;

         DECLARE C_EMBALAGEM CURSOR FOR
            SELECT
               PRODUTO.CDPRODUTO,
               PRODUTO.CDARVPROD,
               ISNULL(PRODUTO.VRFATOCONV, @W_UM) VRFATOCONV,
               ISNULL(PRODUTO.VRARREDCMP, @W_ZERO) VRARREDCMP,
               PERFILPROD.CDPERIENTR
            FROM PRODUTO, PRODUTO PRODPERIENTR, PERFILPROD
            WHERE ((ISNULL(PRODUTO.IDPRCMPATIVO, @W_S)= @W_S) OR (@V_LISTA_ESTOQUE_VAZIA = @W_N))
               AND PRODUTO.CDPRODESTO     = @V_ITEMSOLI_CDPRODUTO
               AND PERFILPROD.CDPRODUTO   = PRODPERIENTR.CDPRODUTO
               AND PERFILPROD.CDFILIAL    = @P_CDFILIAL
               AND ((@V_LISTA_ESTOQUE_VAZIA = @W_S) OR
                    (EXISTS (SELECT 1
                             FROM LISTA
                             WHERE LISTA.NRLISTA = @V_LISTA_ESTOQUE
                               AND LISTA.CDLISTA = PRODUTO.CDPRODUTO))
                   )
               AND ((@V_LISTA_ESTOQUE_VAZIA = @W_N) OR
                    (EXISTS (SELECT 1
                                 FROM PRODFILI
                                 WHERE PRODFILI.CDFILIAL = @P_CDFILIAL
                                   AND PRODFILI.CDPRODUTO = PRODUTO.CDPRODUTO
                                   AND ISNULL(PRODFILI.IDPRODATIV, @W_A)=@W_A))
                   )
               AND PRODPERIENTR.CDARVPROD = (SELECT
                                                MAX(B.CDARVPROD)
                                             FROM
                                                PERFILPROD A,
                                                PRODUTO B
                                             WHERE A.CDPRODUTO = B.CDPRODUTO
                                              AND  A.CDFILIAL  = @P_CDFILIAL
                                              AND  CHARINDEX(B.CDARVPROD, PRODUTO.CDARVPROD) = @W_UM)
            ORDER BY
               ISNULL(PRODUTO.VRFATOCONV, 1)*ISNULL(PRODUTO.VRARREDCMP, 1) DESC;
         DELETE FROM LISTA
         WHERE NRLISTA = @V_LISTA_ESTOQUE;
      END;
      OPEN C_EMBALAGEM;
      FETCH NEXT FROM C_EMBALAGEM INTO @V_EMBALAGEM_CDPRODUTO,  @V_EMBALAGEM_CDARVPROD,
                                       @V_EMBALAGEM_VRFATOCONV, @V_EMBALAGEM_VRARREDCMP,
                                       @V_EMBALAGEM_CDPERIENTR;
      WHILE (@@FETCH_STATUS = 0) BEGIN
         IF @V_PRIMEIRO_REGISTRO = @W_N BEGIN
            FETCH NEXT FROM C_EMBALAGEM INTO @V_EMBALAGEM_CDPRODUTO,  @V_EMBALAGEM_CDARVPROD,
                                             @V_EMBALAGEM_VRFATOCONV, @V_EMBALAGEM_VRARREDCMP,
                                             @V_EMBALAGEM_CDPERIENTR;
         END;
         SET @V_PRIMEIRO_REGISTRO = @W_N;
         IF @V_EMBALAGEM_CDPRODUTO IS NOT NULL BEGIN
            SET @V_COUNT = 0;
            SELECT @V_COUNT = COUNT(1)
              FROM PRODFILI
             WHERE PRODFILI.CDPRODUTO = @V_EMBALAGEM_CDPRODUTO
               AND PRODFILI.CDFILIAL = @P_CDFILIAL;
            IF @V_COUNT > 0 BEGIN
               UPDATE PRODFILI
                  SET IDPRODATIV = @W_A
                WHERE PRODFILI.CDPRODUTO = @V_EMBALAGEM_CDPRODUTO
                  AND PRODFILI.CDFILIAL = @P_CDFILIAL
                  AND PRODFILI.IDPRODATIV = @W_I;
            END      
            ELSE BEGIN
               INSERT INTO PRODFILI (CDFILIAL, CDPRODUTO, IDPRODATIV)
               VALUES(@P_CDFILIAL, @V_EMBALAGEM_CDPRODUTO, @W_A);
            END;

            IF @V_EMBALAGEM_VRARREDCMP > @W_ZERO BEGIN
               SET @V_CONTADOR = @V_EMBALAGEM_VRARREDCMP;
            END
            ELSE BEGIN
               SET @V_CONTADOR = @W_UM;
            END;
            SET @V_QTDEPRODUTO     = @W_ZERO;
            SET @V_AUXQTDERESTANTE = @V_QTDERESTANTE;
            IF (@@FETCH_STATUS <> 0) BEGIN
               WHILE (@V_QTDERESTANTE > @W_ZERO) BEGIN
                  SET @V_QTDEPRODUTO  = @V_CONTADOR;
                  SET @V_QTDERESTANTE = @V_AUXQTDERESTANTE - (@V_EMBALAGEM_VRFATOCONV*@V_CONTADOR);
                  IF @V_EMBALAGEM_VRARREDCMP > @W_ZERO BEGIN
                     SET @V_CONTADOR = @V_CONTADOR + @V_EMBALAGEM_VRARREDCMP;
                  END
                  ELSE BEGIN
                     SET @V_CONTADOR = @V_CONTADOR + @W_UM;
                  END;
               END;
            END
            ELSE BEGIN
               WHILE ((@V_EMBALAGEM_VRFATOCONV*@V_CONTADOR) <= @V_AUXQTDERESTANTE) BEGIN
                  SET @V_QTDEPRODUTO  = @V_CONTADOR;
                  SET @V_QTDERESTANTE = @V_AUXQTDERESTANTE - (@V_EMBALAGEM_VRFATOCONV*@V_CONTADOR);
                  IF @V_EMBALAGEM_VRARREDCMP > @W_ZERO BEGIN
                     SET @V_CONTADOR = @V_CONTADOR + @V_EMBALAGEM_VRARREDCMP;
                  END
                  ELSE BEGIN
                     SET @V_CONTADOR = @V_CONTADOR + @W_UM;
                  END;
               END;
            END;
            IF @V_QTDEPRODUTO > @W_ZERO BEGIN
               SET @V_DTENTREGA = NULL;
               SET @V_NRDIASEG = 0;
               SELECT @V_NRDIASEG = MAX(NRLIMISEGU)
                FROM LIMISEGPROD, PRODUTO, PRODUTO PRODNIVELCOMPL
               WHERE CDFILIAL = @P_CDFILIAL
                 AND LIMISEGPROD.CDPRODUTO = PRODUTO.CDPRODUTO
                 AND PRODNIVELCOMPL.CDPRODUTO = @V_EMBALAGEM_CDPRODUTO
                 AND PRODNIVELCOMPL.CDARVPROD LIKE PRODUTO.CDARVPROD+'%'
                 AND PRODUTO.CDARVPROD = (SELECT MAX(B.CDARVPROD)
                                                FROM LIMISEGPROD A, PRODUTO B
                                               WHERE A.CDPRODUTO = B.CDPRODUTO
                                                 AND A.CDFILIAL  = @P_CDFILIAL
                                                 AND CHARINDEX(B.CDARVPROD, PRODNIVELCOMPL.CDARVPROD) = 1);
               IF @V_NRDIASEG IS NULL BEGIN
                  SET @V_NRDIASEG = 0;
               END;

               SET @V_NOVADATA = @V_ITEMSOLI_DTPROGENTR - @V_NRDIASEG;
               SELECT
                  @V_DTMESENTREGA = @V_NOVADATA - DAY(@V_NOVADATA) + 1;

               SELECT
                  @V_DTENTREGA = MAX(CALECOMP.DTENTREGA)
               FROM
                  CALECOMP
               WHERE CALECOMP.CDFILIAL   =  @P_CDFILIAL
                 AND CALECOMP.CDPERIENTR =  @V_EMBALAGEM_CDPERIENTR
                 AND CALECOMP.DTENTREGA  >= @V_DTMESENTREGA
                 AND CALECOMP.DTENTREGA  <= @V_NOVADATA;

               IF @V_DTENTREGA IS NULL BEGIN
                  EXEC CALC_DATA_ENTR_MES @P_CDFILIAL,
                                          @V_EMBALAGEM_CDPERIENTR,
                                          @V_DTMESENTREGA,
                                          @W_UM,
                                          @V_DTPROCESS;
                  SELECT
                     @V_DTENTREGA = MAX(CALECOMP.DTENTREGA)
                  FROM
                     CALECOMP
                  WHERE CALECOMP.CDFILIAL   =  @P_CDFILIAL
                    AND CALECOMP.CDPERIENTR =  @V_EMBALAGEM_CDPERIENTR
                    AND CALECOMP.DTENTREGA  <= @V_NOVADATA;
               END;

               IF (((@V_DTENTREGA IS NOT NULL) AND (@V_DTENTREGA >= @P_DTINICIAL) AND (@V_DTENTREGA <= @P_DTFINAL) AND (@P_GERASOLOPERLOG = 'S')) OR
                  ((@V_DTENTREGA IS NOT NULL) AND (@P_GERASOLOPERLOG = 'N'))) BEGIN
                  INSERT INTO PRODCONSSOLOL
                     (NRSEQUCONSOL, CDFILSOLI, CDFILLAN, CDPRODUTO, QTESTUTILCONSOL)
                  SELECT
                     @P_NRSEQCONSOL,
                     @P_CDFILIAL,
                     @P_CDFILLANC,
                     @V_EMBALAGEM_CDPRODUTO,
                     @W_ZERO
                  WHERE
                     NOT EXISTS (SELECT 1
                                 FROM PRODCONSSOLOL
                                 WHERE NRSEQUCONSOL    =  @P_NRSEQCONSOL
                                   AND CDFILSOLI       =  @P_CDFILIAL
                                   AND CDFILLAN        =  @P_CDFILLANC
                                   AND CDPRODUTO       =  @V_EMBALAGEM_CDPRODUTO);

                  SET @V_QTSEGURANCA = @W_ZERO;

                  IF (@@ROWCOUNT > 0) BEGIN
                     DECLARE C_ESTOQUE CURSOR FOR
                        SELECT
                           RELACPRODOL.CDPRODARVOL,
                           PRODUTO.CDPRODESTO,
                           RELACPRODOL.CDMARCA
                        FROM
                           RELACPRODOL, PRODUTO
                        WHERE RELACPRODOL.CDPRARVORIG = @V_EMBALAGEM_CDPRODUTO
                          AND PRODUTO.CDPRODUTO       = RELACPRODOL.CDPRODARVOL
                     OPEN C_ESTOQUE;
                     FETCH NEXT FROM C_ESTOQUE INTO @V_ESTOQUE_CDPRODARVOL,
                                                    @V_ESTOQUE_CDPRODESTO,
                                                    @V_ESTOQUE_CDMARCA;
                     WHILE (@@FETCH_STATUS = 0) BEGIN
                        SET @V_QTESTOQUENECCMP = @W_ZERO;
                        SET @V_QTDE_ENTRADA    = @W_ZERO;
                        SET @V_QTDE_SAIDA      = @W_ZERO;
                        IF @V_ESTOQUE_CDPRODARVOL = @V_ESTOQUE_CDPRODESTO BEGIN
                           SELECT
                              @V_QTESTOQUENECCMP = TEKNISA.CALC_ESTOQ_CONS(@P_CDFILIAL, @V_ESTOQUE_CDPRODARVOL, @V_DTPROCESS_MAIS_UM);
                           SELECT
                              @V_QTDE_ENTRADA = SUM(ITEMAUTF.QTPROGENTAF)
                           FROM
                              ITEMAUTF
                           WHERE ITEMAUTF.CDFILDEST   = @P_CDFILIAL
                             AND ITEMAUTF.DTPROGENTAF BETWEEN @V_DTPROCESS AND @P_DTFINAL
                             AND ITEMAUTF.CDPRODUTO   = @V_ESTOQUE_CDPRODARVOL
                             AND ITEMAUTF.IDBAIXITMAF  IS NULL
                             AND EXISTS (SELECT
                                             @W_UM
                                          FROM
                                             AUTOFORNE
                                          WHERE AUTOFORNE.CDFILLANC  = ITEMAUTF.CDFILLANC
                                            AND AUTOFORNE.CDFILDEST  = ITEMAUTF.CDFILDEST
                                            AND AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN
                                            AND ISNULL(AUTOFORNE.IDBAIXAAF, '1') = '1');
                           SELECT @V_QTESTOQUESEGURANCA = NVL(MAX(QTESTOSEGU), 0)
                             FROM ESTOSEGU
                            WHERE CDFILIAL   = @P_CDFILIAL
                              AND CDPRODESTO = @V_ESTOQUE_CDPRODARVOL;

                            IF (@V_QTESTOQUENECCMP < @V_QTESTOQUESEGURANCA) THEN
                               SET @V_QTSEGURANCA := @V_QTESTOQUESEGURANCA - @V_QTESTOQUENECCMP;
                            END IF;
                              PETTERSON
                           IF @P_PREVISAOCONSUMO = @W_N BEGIN
                              SELECT
                                 @V_QTDE_SAIDA = SUM(ISNULL(ITEMPEDEXOL.QTATENDPEDOL, ITEMPEDEXOL.QTPROGENPEOL))
                              FROM
                                 PEDIDOEXOL,
                                 ITEMPEDEXOL
                              WHERE ITEMPEDEXOL.CDPRODUTO     = @V_EMBALAGEM_CDPRODUTO
                                AND ITEMPEDEXOL.CDMARCA       = @V_ESTOQUE_CDMARCA
                                AND ITEMPEDEXOL.IDBAIXAOLSAI  IS NULL
                                AND ITEMPEDEXOL.DTPROGEMBARQ  BETWEEN @V_DTPROCESS AND @P_DTFINAL
                                AND ITEMPEDEXOL.CDFILDEST     = PEDIDOEXOL.CDFILDEST
                                AND ITEMPEDEXOL.CDFILLANC     = PEDIDOEXOL.CDFILLANC
                                AND ITEMPEDEXOL.NRPEDEXOL     = PEDIDOEXOL.NRPEDEXOL
                                AND ISNULL(ITEMPEDEXOL.QTATENDPEDOL, 1) > 0
                                AND PEDIDOEXOL.CDFILIOPELOG   = @P_CDFILIAL;

                              SET @V_QTESTOQUENECCMP = ISNULL(@V_QTESTOQUENECCMP, @W_ZERO);
                              SET @V_QTDE_ENTRADA    = ISNULL(@V_QTDE_ENTRADA, @W_ZERO);
                              SET @V_QTDE_SAIDA      = ISNULL(@V_QTDE_SAIDA, @W_ZERO);
                              SET @V_QTESTOQUENECCMP = @V_QTESTOQUENECCMP + @V_QTDE_ENTRADA - @V_QTDE_SAIDA;
                           END
                           ELSE BEGIN
                              SET @V_MAX_LIMISEGU = 0;
                              SELECT @V_MAX_LIMISEGU = ISNULL(MAX(CARREGA_LIMISEGU(@V_ITEMSOLI_CDFILSOLI, CDARVPROD, @P_DTFINAL)), 0)
                                FROM PRODUTO
                               WHERE CDPRODUTO = @V_EMBALAGEM_CDPRODUTO
                                 AND CDPRODESTO IS NOT NULL;
                              SELECT
                                 (SUM(TRABNECECOMP.QTPRCNVCOMPR*PRODUTO.VRFATOCONV)/ @V_EMBALAGEM_VRFATOCONV)
                              INTO V_QTDE_SAIDA
                              FROM TRABNECECOMP, PRODUTO, PRODUTO PRODESTO, PRODUTO PRODCMP
                              WHERE TRABNECECOMP.DTENTRNECCMP BETWEEN @V_DTPROCESS AND @P_DTINICIAL+@V_MAX_LIMISEGU_OL
                                AND PRODUTO.CDPRODUTO         = TRABNECECOMP.CDPRODNECCMP
                                AND PRODUTO.CDPRODESTO        = PRODESTO.CDPRODUTO
                                AND PRODCMP.CDPRODUTO         = @V_EMBALAGEM_CDPRODUTO
                                AND PRODCMP.CDPRODESTO        = PRODESTO.CDPRODUTO
                                AND TRABNECECOMP.CDFILNECECMP <> @P_CDFILIAL
                                AND (TRABNECECOMP.NRNECECOMP = @V_PRXNECECOMP)
                                AND (TRABNECECOMP.IDMARCNECCMP = @W_S)
                                AND (TRABNECECOMP.CDFILIOPELOG = @P_CDFILIAL);

                              SET @V_QTESTOQUENECCMP = ISNULL(@V_QTESTOQUENECCMP, @W_ZERO);
                              SET @V_QTDE_ENTRADA    = ISNULL(@V_QTDE_ENTRADA, @W_ZERO);
                              SET @V_QTDE_SAIDA      = ISNULL(@V_QTDE_SAIDA, @W_ZERO);
                              SET @V_QTESTOQUENECCMP = @V_QTESTOQUENECCMP + @V_QTDE_ENTRADA - @V_QTDE_SAIDA;

                           END;
                           IF @V_QTESTOQUENECCMP > @W_ZERO BEGIN
                              INSERT INTO
                                 ESTOCONSSOLOL (NRSEQUCONSOL, CDFILSOLI, CDFILLAN,
                                                CDPRODUTO, CDPRODARVOL, QTESTOPRODOL,
                                                QTORIGPRODOL, IDUTESTCONSOL,
                                                QTENTRCONSOL, QTSAIDACONSOL, DTCONSESTCONSOL)
                              SELECT
                                 @P_NRSEQCONSOL,
                                 @P_CDFILIAL,
                                 @P_CDFILLANC,
                                 @V_EMBALAGEM_CDPRODUTO,
                                 @V_ESTOQUE_CDPRODARVOL,
                                 @V_QTESTOQUENECCMP,
                                 @V_QTESTOQUENECCMP,
                                 @W_N,
                                 @V_QTDE_ENTRADA,
                                 @V_QTDE_SAIDA,
                                 @V_DTPROCESS
                              WHERE
                                 NOT EXISTS (SELECT @W_UM
                                             FROM ESTOCONSSOLOL
                                             WHERE NRSEQUCONSOL = @P_NRSEQCONSOL
                                               AND CDFILSOLI    = @P_CDFILIAL
                                               AND CDFILLAN     = @P_CDFILLANC
                                               AND CDPRODUTO    = @V_EMBALAGEM_CDPRODUTO
                                               AND CDPRODARVOL  = @V_ESTOQUE_CDPRODARVOL);
                           END;
                        END
                        ELSE BEGIN
                           SELECT
                              @V_QTESTOQUENECCMP = TEKNISA.CALC_ESTOQ_CONS(@P_CDFILIAL, @V_ESTOQUE_CDPRODESTO, @V_DTPROCESS_MAIS_UM);
                           SELECT
                              @V_QTDE_ENTRADA = SUM(ITEMAUTF.QTPROGENTAF*ISNULL(PRODUTO.VRFATOCONV,1))
                           FROM
                              ITEMAUTF, PRODUTO
                           WHERE ITEMAUTF.CDFILDEST   = @P_CDFILIAL
                             AND ITEMAUTF.DTPROGENTAF BETWEEN @V_DTPROCESS AND @P_DTFINAL
                             AND ITEMAUTF.CDPRODUTO   = PRODUTO.CDPRODUTO
                             AND PRODUTO.CDPRODESTO   = @V_ESTOQUE_CDPRODESTO
                             AND ITEMAUTF.IDBAIXITMAF IS NULL
                             AND EXISTS (SELECT
                                             @W_UM
                                          FROM
                                             AUTOFORNE
                                          WHERE AUTOFORNE.CDFILLANC  = ITEMAUTF.CDFILLANC
                                            AND AUTOFORNE.CDFILDEST  = ITEMAUTF.CDFILDEST
                                            AND AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN
                                            AND ISNULL(AUTOFORNE.IDBAIXAAF, '1') = '1');
                           IF @P_PREVISAOCONSUMO = @W_N BEGIN
                              SELECT
                                 @V_QTDE_SAIDA = SUM(ISNULL(ITEMPEDEXOL.QTATENDPEDOL, ITEMPEDEXOL.QTPROGENPEOL))
                              FROM
                                 PEDIDOEXOL,
                                 ITEMPEDEXOL
                              WHERE ITEMPEDEXOL.CDPRODUTO     = @V_EMBALAGEM_CDPRODUTO
                                AND ITEMPEDEXOL.CDMARCA       = @V_ESTOQUE_CDMARCA
                                AND ITEMPEDEXOL.IDBAIXAOLSAI  IS NULL
                                AND ITEMPEDEXOL.DTPROGEMBARQ  BETWEEN @V_DTPROCESS AND @P_DTFINAL
                                AND ITEMPEDEXOL.CDFILDEST     = PEDIDOEXOL.CDFILDEST
                                AND ITEMPEDEXOL.CDFILLANC     = PEDIDOEXOL.CDFILLANC
                                AND ITEMPEDEXOL.NRPEDEXOL     = PEDIDOEXOL.NRPEDEXOL
                                AND ISNULL(ITEMPEDEXOL.QTATENDPEDOL, 1) > 0
                                AND PEDIDOEXOL.CDFILIOPELOG   = @P_CDFILIAL;

                              SET @V_QTESTOQUENECCMP = ISNULL(@V_QTESTOQUENECCMP, @W_ZERO);
                              SET @V_QTDE_ENTRADA    = ISNULL(@V_QTDE_ENTRADA, @W_ZERO);
                              SET @V_QTDE_SAIDA      = ISNULL(@V_QTDE_SAIDA, @W_ZERO);
                              SET @V_QTESTOQUENECCMP = @V_QTESTOQUENECCMP + @V_QTDE_ENTRADA - @V_QTDE_SAIDA;
                           END
                           ELSE BEGIN
                              SET @V_MAX_LIMISEGU = 0;
                              SELECT @V_MAX_LIMISEGU = ISNULL(MAX(CARREGA_LIMISEGU(@V_ITEMSOLI_CDFILSOLI, CDARVPROD, @P_DTFINAL)), 0)
                                FROM PRODUTO
                               WHERE CDPRODUTO = @V_EMBALAGEM_CDPRODUTO
                                 AND CDPRODESTO IS NOT NULL;
                              SELECT
                                 (SUM(TRABNECECOMP.QTPRCNVCOMPR*PRODUTO.VRFATOCONV)/ @V_EMBALAGEM_VRFATOCONV)
                              INTO V_QTDE_SAIDA
                              FROM TRABNECECOMP, PRODUTO, PRODUTO PRODESTO, PRODUTO PRODCMP
                              WHERE TRABNECECOMP.DTENTRNECCMP BETWEEN @V_DTPROCESS AND @P_DTINICIAL+@V_MAX_LIMISEGU_OL
                                AND PRODUTO.CDPRODUTO         = TRABNECECOMP.CDPRODNECCMP
                                AND PRODUTO.CDPRODESTO        = PRODESTO.CDPRODUTO
                                AND PRODCMP.CDPRODUTO         = @V_EMBALAGEM_CDPRODUTO
                                AND PRODCMP.CDPRODESTO        = PRODESTO.CDPRODUTO
                                AND TRABNECECOMP.CDFILNECECMP <> @P_CDFILIAL
                                AND (TRABNECECOMP.NRNECECOMP = @V_PRXNECECOMP)
                                AND (TRABNECECOMP.IDMARCNECCMP = @W_S)
                                AND (TRABNECECOMP.CDFILIOPELOG = @P_CDFILIAL);

                              SET @V_QTESTOQUENECCMP = ISNULL(@V_QTESTOQUENECCMP, @W_ZERO);
                              SET @V_QTDE_ENTRADA    = ISNULL(@V_QTDE_ENTRADA, @W_ZERO);
                              SET @V_QTDE_SAIDA      = ISNULL(@V_QTDE_SAIDA, @W_ZERO);
                              SET @V_QTESTOQUENECCMP = @V_QTESTOQUENECCMP + @V_QTDE_ENTRADA - @V_QTDE_SAIDA;

                           END;
                           IF @V_QTESTOQUENECCMP > @W_ZERO BEGIN
                              INSERT INTO
                                 ESTOCONSSOLOL (NRSEQUCONSOL, CDFILSOLI, CDFILLAN,
                                                CDPRODUTO, CDPRODARVOL, QTESTOPRODOL,
                                                QTORIGPRODOL, IDUTESTCONSOL,
                                                QTENTRCONSOL, QTSAIDACONSOL, DTCONSESTCONSOL)
                              SELECT
                                 @P_NRSEQCONSOL,
                                 @P_CDFILIAL,
                                 @P_CDFILLANC,
                                 @V_EMBALAGEM_CDPRODUTO,
                                 @V_ESTOQUE_CDPRODESTO,
                                 @V_QTESTOQUENECCMP,
                                 @V_QTESTOQUENECCMP,
                                 @W_N,
                                 @V_QTDE_ENTRADA,
                                 @V_QTDE_SAIDA,
                                 @V_DTPROCESS
                              WHERE
                                 NOT EXISTS (SELECT @W_UM
                                             FROM ESTOCONSSOLOL
                                             WHERE NRSEQUCONSOL = @P_NRSEQCONSOL
                                               AND CDFILSOLI    = @P_CDFILIAL
                                               AND CDFILLAN     = @P_CDFILLANC
                                               AND CDPRODUTO    = @V_EMBALAGEM_CDPRODUTO
                                               AND CDPRODARVOL  = @V_ESTOQUE_CDPRODARVOL);
                           END;
                        END;
                        FETCH NEXT FROM C_ESTOQUE INTO @V_ESTOQUE_CDPRODARVOL,
                                                       @V_ESTOQUE_CDPRODESTO,
                                                       @V_ESTOQUE_CDMARCA;
                     END;
                     CLOSE C_ESTOQUE;
                     DEALLOCATE C_ESTOQUE;
                  END;
                  SET @V_QTDEPRODUTO = @V_QTDEPRODUTO * (1 + (@P_PEACRESCIMO / 100));
                  UPDATE ITEMCONSSOLOL
                  SET
                     QTENTRCONSOL   = QTENTRCONSOL + @V_QTDEPRODUTO,
                     QTESTOCONSOL   = @W_ZERO,
                     IDENTREGAVIAOL = @W_S
                  WHERE NRSEQUCONSOL =  @P_NRSEQCONSOL
                    AND CDFILSOLI    =  @P_CDFILIAL
                    AND CDFILLAN     =  @P_CDFILLANC
                    AND CDPRODUTO    =  @V_EMBALAGEM_CDPRODUTO
                    AND DTENTRCONSOL =  @V_DTENTREGA;

                  IF (@@ROWCOUNT <= 0) BEGIN
                     INSERT INTO ITEMCONSSOLOL
                        (NRSEQUCONSOL, CDFILSOLI, CDFILLAN, CDPRODUTO, DTENTRCONSOL,
                         QTENTRCONSOL, QTESTOCONSOL, IDENTREGAVIAOL)
                     VALUES
                        (@P_NRSEQCONSOL, @P_CDFILIAL, @P_CDFILLANC, @V_EMBALAGEM_CDPRODUTO,
                         @V_DTENTREGA, @V_QTDEPRODUTO+@V_QTSEGURANCA, @W_ZERO, @W_S);
                  END;

                  IF @P_PREVISAOCONSUMO = @W_N BEGIN
                     UPDATE RELACONSOLOL
                     SET QTENTRCONS = QTENTRCONS + @V_QTDEPRODUTO
                     WHERE NRSEQUCONSOL = @P_NRSEQCONSOL
                       AND CDFILSOLCONS =  @P_CDFILIAL
                       AND CDFILLANCONS =  @P_CDFILLANC
                       AND CDPRODUTOCONS =  @V_EMBALAGEM_CDPRODUTO
                       AND DTENTRCONSOL =  @V_DTENTREGA
                       AND CDPRODUTO  IN (SELECT PRODUTO.CDPRODUTO
                                          FROM PRODUTO
                                          WHERE CDPRODESTO = @V_ITEMSOLI_CDPRODUTO)
                       AND CDFILSOLI = @V_ITEMSOLI_CDFILSOLI
                       AND DTPROGENTR = @V_ITEMSOLI_DTPROGENTR;

                     IF (@@ROWCOUNT = 0) BEGIN
                        INSERT INTO RELACONSOLOL
                           (CDFILSOLI,CDFILLAN,NRSOLICMP,CDPRODUTO,DTPROGENTR,
                            NRSEQUCONSOL,CDFILSOLCONS,CDFILLANCONS,CDPRODUTOCONS,DTENTRCONSOL, QTENTRCONS)
                        SELECT
                           ITEMSOLI.CDFILSOLI,
                           ITEMSOLI.CDFILLAN,
                           ITEMSOLI.NRSOLICMP,
                           ITEMSOLI.CDPRODUTO,
                           ITEMSOLI.DTPROGENTR,
                           ITEMCONSSOLOL.NRSEQUCONSOL,
                           ITEMCONSSOLOL.CDFILSOLI,
                           ITEMCONSSOLOL.CDFILLAN,
                           ITEMCONSSOLOL.CDPRODUTO,
                           ITEMCONSSOLOL.DTENTRCONSOL,
                           @V_QTDEPRODUTO
                        FROM
                           ITEMCONSSOLOL, ITEMSOLI
                        WHERE ITEMSOLI.CDFILSOLI     = @V_ITEMSOLI_CDFILSOLI
                          AND ITEMSOLI.DTPROGENTR    = @V_ITEMSOLI_DTPROGENTR
                          AND ((ITEMSOLI.CDFILIOPELOG  = @P_CDFILIAL) OR (@P_CDOPERADOR IS NULL))
                          AND ITEMSOLI.CDFILSOLI    <> @P_CDFILIAL
                          AND ITEMSOLI.NRSEQUCONSOL  IS NULL
                          AND ISNULL(ITEMSOLI.IDSOLIAPROV, @W_S) = @W_S
                          AND ISNULL(ITEMSOLI.IDITMCONS, @W_N) = @W_N
                          AND ITEMSOLI.CDPRODUTO  IN (SELECT PRODUTO.CDPRODUTO
                                                      FROM PRODUTO
                                                      WHERE CDPRODESTO = @V_ITEMSOLI_CDPRODUTO)
                          AND ITEMCONSSOLOL.NRSEQUCONSOL =  @P_NRSEQCONSOL
                          AND ITEMCONSSOLOL.CDFILSOLI    =  @P_CDFILIAL
                          AND ITEMCONSSOLOL.CDFILLAN     =  @P_CDFILLANC
                          AND ITEMCONSSOLOL.CDPRODUTO    =  @V_EMBALAGEM_CDPRODUTO
                          AND ITEMCONSSOLOL.DTENTRCONSOL =  @V_DTENTREGA
                          AND NOT EXISTS (SELECT 1
                                          FROM RELACONSOLOL R
                                          WHERE R.CDFILSOLI     = ITEMSOLI.CDFILSOLI
                                            AND R.CDFILLAN      = ITEMSOLI.CDFILLAN
                                            AND R.NRSOLICMP     = ITEMSOLI.NRSOLICMP
                                            AND R.CDPRODUTO     = ITEMSOLI.CDPRODUTO
                                            AND R.DTPROGENTR    = ITEMSOLI.DTPROGENTR
                                            AND R.NRSEQUCONSOL  = ITEMCONSSOLOL.NRSEQUCONSOL
                                            AND R.CDFILSOLCONS  = ITEMCONSSOLOL.CDFILSOLI
                                            AND R.CDFILLANCONS  = ITEMCONSSOLOL.CDFILLAN
                                            AND R.CDPRODUTOCONS = ITEMCONSSOLOL.CDPRODUTO
                                            AND R.DTENTRCONSOL  = ITEMCONSSOLOL.DTENTRCONSOL);
                     END;
                  END
                  ELSE BEGIN
                     UPDATE RELACONSOLOLNC
                     SET QTNCPREVCONSOL  = QTNCPREVCONSOL + @V_QTDEPRODUTO
                     WHERE NRSEQUCONSOL  = @P_NRSEQCONSOL
                       AND CDFILSOLCONS  = @P_CDFILIAL
                       AND CDFILLANCONS  = @P_CDFILLANC
                       AND CDPRODUTOCONS = @V_EMBALAGEM_CDPRODUTO
                       AND DTENTRCONSOL  = @V_DTENTREGA
                       AND CDPRODUTO     = @V_ITEMSOLI_CDPRODUTO
                       AND CDFILIAL      = @V_ITEMSOLI_CDFILSOLI
                       AND NRNCPREVCONSOL= @V_PRXNECECOMP
                       AND DTNCPREVCONSOL= @V_ITEMSOLI_DTPROGENTR;

                     IF (@@ROWCOUNT = 0) BEGIN
                        INSERT INTO RELACONSOLOLNC
                           (NRSEQUCONSOL,CDFILSOLCONS,CDFILLANCONS,CDPRODUTOCONS,
                            DTENTRCONSOL,CDPRODUTO,CDFILIAL,NRNCPREVCONSOL,DTNCPREVCONSOL,
                            QTNCPREVCONSOL)
                        SELECT
                           @P_NRSEQCONSOL,
                           @P_CDFILIAL,
                           @P_CDFILLANC,
                           @V_EMBALAGEM_CDPRODUTO,
                           @V_DTENTREGA,
                           @V_ITEMSOLI_CDPRODUTO,
                           @V_ITEMSOLI_CDFILSOLI,
                           @V_PRXNECECOMP,
                           @V_ITEMSOLI_DTPROGENTR,
                           @V_QTDEPRODUTO
                        WHERE
                          NOT EXISTS (SELECT 1
                                      FROM RELACONSOLOLNC R
                                      WHERE R.NRSEQUCONSOL   = @P_NRSEQCONSOL
                                        AND R.CDFILSOLCONS   = @P_CDFILIAL
                                        AND R.CDFILLANCONS   = @P_CDFILLANC
                                        AND R.CDPRODUTOCONS  = @V_EMBALAGEM_CDPRODUTO
                                        AND R.DTENTRCONSOL   = @V_DTENTREGA
                                        AND R.CDPRODUTO      = @V_ITEMSOLI_CDPRODUTO
                                        AND R.CDFILIAL       = @V_ITEMSOLI_CDFILSOLI
                                        AND R.NRNCPREVCONSOL = @V_PRXNECECOMP
                                        AND R.DTNCPREVCONSOL = @V_ITEMSOLI_DTPROGENTR);
                     END;
                  END;
                  SET @V_ITEM_CONSOLIDADO = @W_S;
               END;
            END;
         END;
      END;
      CLOSE C_EMBALAGEM;
      DEALLOCATE C_EMBALAGEM;
      IF @V_ITEM_CONSOLIDADO = @W_S BEGIN
         IF @P_PREVISAOCONSUMO = @W_N BEGIN
            UPDATE ITEMSOLI
            SET NRSEQUCONSOL = @P_NRSEQCONSOL,
                CDFILIOPELOG = @P_CDFILIAL
            WHERE ITEMSOLI.CDFILSOLI    = @V_ITEMSOLI_CDFILSOLI
              AND ITEMSOLI.DTPROGENTR   = @V_ITEMSOLI_DTPROGENTR
              AND ((ITEMSOLI.CDFILIOPELOG  = @P_CDFILIAL) OR (@P_CDOPERADOR IS NULL))
              AND ITEMSOLI.CDFILSOLI    <> @P_CDFILIAL
              AND ISNULL(ITEMSOLI.IDSOLIAPROV, @W_S) = @W_S
              AND ISNULL(ITEMSOLI.IDITMCONS, @W_N) = @W_N
              AND ITEMSOLI.NRSEQUCONSOL  IS NULL
              AND ITEMSOLI.CDPRODUTO  IN (SELECT PRODUTO.CDPRODUTO
                                          FROM PRODUTO
                                          WHERE CDPRODESTO = @V_ITEMSOLI_CDPRODUTO);
         END;
         SET @P_ROWSAFFECT = @W_S;
      END;
      FETCH NEXT FROM C_ITEMSOLI_AUX INTO @V_ITEMSOLI_CDFILSOLI,
                                      @V_ITEMSOLI_CDPRODUTO,
                                      @V_ITEMSOLI_CDARVPROD,
                                      @V_ITEMSOLI_DTPROGENTR,
                                      @V_ITEMSOLI_QTSOLIPROD;
   END;
   CLOSE C_ITEMSOLI_AUX;
   DEALLOCATE C_ITEMSOLI_AUX;
   IF @V_IDUTEMPENCONSOL = @W_S BEGIN
      IF @P_CDOPERADOR IS NOT NULL BEGIN
         EXEC EMPENHO_OL @P_NRSEQCONSOL, @P_CDFILIAL, @V_CDFILANTERIOR, @V_CDPRODANTERIOR, @P_CDOPERADOR, @V_PRXNECECOMP;
      END
      ELSE BEGIN
         EXEC EMPENHO_OL @P_NRSEQCONSOL, @P_CDFILIAL, @V_CDFILANTERIOR, @V_CDPRODANTERIOR, @V_CDOPERADOR, @V_PRXNECECOMP;
      END;
   END;

   IF (@P_ROWSAFFECT = @W_N) AND (@P_CDOPERADOR IS NOT NULL) BEGIN
      DELETE FROM CONSSOLOL
      WHERE NRSEQUCONSOL = @P_NRSEQCONSOL;
   END;
   IF (@V_PRXNECECOMP IS NOT NULL) AND NOT(@P_PREVISAOCONSUMO = 'W_N') BEGIN
      DELETE FROM TRABNECECOMP
      WHERE CDFILIOPELOG = @P_CDFILIAL
        AND NRNECECOMP = @V_PRXNECECOMP
        AND DTENTRNECCMP BETWEEN @P_DTINICIAL AND @P_DTFINAL
        AND CDPRODNECCMP IN (
          SELECT CDPRODUTO FROM PRODUTO
          WHERE CDARVPROD BETWEEN @P_CDPRODINI AND @V_CDPRODFIN)
        AND ISNULL(QTTEORNECCMP, 0) = 0;
   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'NECESSIDADE_OL','5.80.XXX';
GO
